@@ -24,6 +24,7 @@
 #include <stdint.h>
 #include <linux/types.h>
 #include <stdbool.h>
+#include <assert.h>
 
 #include "ica_api.h"
 #include "icastats.h"
@@ -522,6 +523,249 @@ unsigned int ica_sha512(unsigned int message_part,
 			   (uint64_t *) &sha512_context->runningLengthHigh);
 }
 
+unsigned int ica_sha3_224(unsigned int message_part,
+			unsigned int input_length,
+			unsigned char *input_data,
+			sha3_224_context_t *sha3_224_context,
+			unsigned char *output_data)
+{
+	unsigned int rc;
+
+#ifdef ICA_FIPS
+	if (fips >> 1)
+		return EACCES;
+#endif /* ICA_FIPS */
+
+	/* check for obvious errors in parms */
+	if ((input_data == NULL) ||
+	    (sha3_224_context == NULL) ||
+	    (output_data == NULL))
+		return EINVAL;
+
+	/* make sure some message part is specified */
+	rc = check_message_part(message_part);
+	if (rc)
+		return rc;
+
+	/*
+	 * for FIRST or MIDDLE calls the input
+	 * data length must be a multiple of 144 bytes.
+	 */
+	if ((input_length % 144 != 0) &&
+	    (message_part == SHA_MSG_PART_FIRST ||
+	     message_part == SHA_MSG_PART_MIDDLE))
+		return EINVAL;
+
+	return s390_sha3_224((unsigned char *) &sha3_224_context->sha3_224Hash,
+			   input_data, input_length, output_data, message_part,
+			   (uint64_t *)&sha3_224_context->runningLength);
+}
+
+unsigned int ica_sha3_256(unsigned int message_part,
+			unsigned int input_length,
+			unsigned char *input_data,
+			sha3_256_context_t *sha3_256_context,
+			unsigned char *output_data)
+{
+	unsigned int rc;
+
+#ifdef ICA_FIPS
+	if (fips >> 1)
+		return EACCES;
+#endif /* ICA_FIPS */
+
+	/* check for obvious errors in parms */
+	if ((input_data == NULL) ||
+	    (sha3_256_context == NULL) ||
+	    (output_data == NULL))
+		return EINVAL;
+
+	/* make sure some message part is specified */
+	rc = check_message_part(message_part);
+	if (rc)
+		return rc;
+
+	/*
+	 * for FIRST or MIDDLE calls the input
+	 * data length must be a multiple of 136 bytes.
+	 */
+	if ((input_length % 136 != 0) &&
+	    (message_part == SHA_MSG_PART_FIRST ||
+	     message_part == SHA_MSG_PART_MIDDLE))
+		return EINVAL;
+
+	return s390_sha3_256((unsigned char *) &sha3_256_context->sha3_256Hash,
+			   input_data, input_length, output_data, message_part,
+			   (uint64_t *) &sha3_256_context->runningLength);
+}
+
+unsigned int ica_sha3_384(unsigned int message_part,
+			uint64_t input_length,
+			unsigned char *input_data,
+			sha3_384_context_t *sha3_384_context,
+			unsigned char *output_data)
+{
+	unsigned int rc;
+
+#ifdef ICA_FIPS
+	if (fips >> 1)
+		return EACCES;
+#endif /* ICA_FIPS */
+
+	/* check for obvious errors in parms */
+	if ((input_data == NULL) ||
+	    (sha3_384_context == NULL) ||
+	    (output_data == NULL))
+		return EINVAL;
+
+	/* make sure some message part is specified */
+	rc = check_message_part(message_part);
+	if (rc)
+		return rc;
+
+	/*
+	 * for FIRST or MIDDLE calls the input
+	 * data length must be a multiple of 104 bytes.
+	 */
+	if ((input_length % 104 != 0) &&
+	    (message_part == SHA_MSG_PART_FIRST ||
+	     message_part == SHA_MSG_PART_MIDDLE))
+		return EINVAL;
+
+	return s390_sha3_384((unsigned char *) &sha3_384_context->sha3_384Hash,
+			   input_data, input_length, output_data, message_part,
+			   (uint64_t *) &(sha3_384_context->runningLengthLow),
+			   (uint64_t *) &(sha3_384_context->runningLengthHigh));
+}
+
+unsigned int ica_sha3_512(unsigned int message_part,
+			uint64_t input_length,
+			unsigned char *input_data,
+			sha3_512_context_t *sha3_512_context,
+			unsigned char *output_data)
+{
+	unsigned int rc;
+
+#ifdef ICA_FIPS
+	if (fips >> 1)
+		return EACCES;
+#endif /* ICA_FIPS */
+
+	/* check for obvious errors in parms */
+	if ((input_data == NULL) ||
+	    (sha3_512_context == NULL) ||
+	    (output_data == NULL))
+		return EINVAL;
+
+	/* make sure some message part is specified */
+	rc = check_message_part(message_part);
+	if (rc)
+		return rc;
+
+	/*
+	 * for FIRST or MIDDLE calls the input
+	 * data length must be a multiple of 72 bytes.
+	 */
+	if ((input_length % 72 != 0) &&
+	    (message_part == SHA_MSG_PART_FIRST ||
+	     message_part == SHA_MSG_PART_MIDDLE))
+		return EINVAL;
+
+	return s390_sha3_512((unsigned char *)&sha3_512_context->sha3_512Hash,
+			   input_data, input_length, output_data, message_part,
+			   (uint64_t *) &sha3_512_context->runningLengthLow,
+			   (uint64_t *) &sha3_512_context->runningLengthHigh);
+}
+
+unsigned int ica_shake_128(unsigned int message_part,
+			uint64_t input_length,
+			unsigned char *input_data,
+			shake_128_context_t *shake_128_context,
+			unsigned char *output_data, unsigned int output_length)
+{
+	unsigned int rc;
+
+#ifdef ICA_FIPS
+	if (fips >> 1)
+		return EACCES;
+#endif /* ICA_FIPS */
+
+	/* check for obvious errors in parms */
+	if ((input_data == NULL) ||
+	    (shake_128_context == NULL) ||
+	    (output_length == 0) ||
+	    (output_data == NULL))
+		return EINVAL;
+
+	/* make sure some message part is specified */
+	rc = check_message_part(message_part);
+	if (rc)
+		return rc;
+
+	/*
+	 * for FIRST or MIDDLE calls the input
+	 * data length must be a multiple of 168 bytes.
+	 */
+	if ((input_length % 168 != 0) &&
+	    (message_part == SHA_MSG_PART_FIRST ||
+	     message_part == SHA_MSG_PART_MIDDLE)) {
+		return EINVAL;
+	}
+
+	/* set output_length in context for first call and only call */
+	if ((message_part == SHA_MSG_PART_FIRST || message_part == SHA_MSG_PART_ONLY))
+		shake_128_context->output_length = output_length;
+
+	return s390_shake_128((unsigned char *)&shake_128_context->shake_128Hash,
+			   input_data, input_length, output_data, shake_128_context->output_length,
+			   message_part, (uint64_t *) &shake_128_context->runningLengthLow,
+			   (uint64_t *) &shake_128_context->runningLengthHigh);
+}
+
+unsigned int ica_shake_256(unsigned int message_part,
+			uint64_t input_length,
+			unsigned char *input_data,
+			shake_256_context_t *shake_256_context,
+			unsigned char *output_data, unsigned int output_length)
+{
+	unsigned int rc;
+
+#ifdef ICA_FIPS
+	if (fips >> 1)
+		return EACCES;
+#endif /* ICA_FIPS */
+
+	/* check for obvious errors in parms */
+	if ((input_data == NULL) ||
+	    (shake_256_context == NULL) ||
+	    (output_length == 0) ||
+	    (output_data == NULL))
+		return EINVAL;
+
+	/* make sure some message part is specified */
+	rc = check_message_part(message_part);
+	if (rc)
+		return rc;
+
+	/*
+	 * for FIRST or MIDDLE calls the input
+	 * data length must be a multiple of 136 bytes.
+	 */
+	if ((input_length % 136 != 0) &&
+	    (message_part == SHA_MSG_PART_FIRST ||
+	     message_part == SHA_MSG_PART_MIDDLE))
+		return EINVAL;
+
+	/* set output_length in context for first call and only call */
+	if ((message_part == SHA_MSG_PART_FIRST || message_part == SHA_MSG_PART_ONLY))
+		shake_256_context->output_length = output_length;
+
+	return s390_shake_256((unsigned char *)&shake_256_context->shake_256Hash,
+			   input_data, input_length, output_data, shake_256_context->output_length,
+			   message_part, (uint64_t *) &shake_256_context->runningLengthLow,
+			   (uint64_t *) &shake_256_context->runningLengthHigh);
+}
+
 unsigned int ica_random_number_generate(unsigned int output_length,
 					unsigned char *output_data)
 {
@@ -627,6 +871,9 @@ unsigned int ica_rsa_mod_expo(ica_adapter_handle_t adapter_handle,
 	if (input_data == NULL || rsa_key == NULL || output_data == NULL)
 		return EINVAL;
 
+	if (rsa_key->key_length < sizeof(unsigned long))
+		return EINVAL;
+
 	/* fill driver structure */
 	rb.inputdata = (char *)input_data;
 	rb.inputdatalength = rsa_key->key_length;
@@ -725,6 +972,9 @@ unsigned int ica_rsa_crt(ica_adapter_handle_t adapter_handle,
 	if (input_data == NULL || rsa_key == NULL || output_data == NULL)
 		return EINVAL;
 
+	if (rsa_key->key_length < sizeof(unsigned long))
+		return EINVAL;
+
 	/* fill driver structure */
 	rb.inputdata = (char *)input_data;
 	rb.inputdatalength = rsa_key->key_length;
@@ -1750,12 +2000,21 @@ unsigned int ica_aes_gcm(unsigned char *plaintext, unsigned long plaintext_lengt
 		return EACCES;
 #endif /* ICA_FIPS */
 
-	if (check_aes_parms(MODE_GCM, plaintext_length, plaintext, iv, key_length,
-			    key, ciphertext))
-		return EINVAL;
+	if (plaintext_length != 0) {
+		if (check_aes_parms(MODE_GCM, plaintext_length, plaintext, iv, key_length,
+				    key, ciphertext))
+			return EINVAL;
+	} else {
+		/* If only aad is processed (ghash), pt/ct may be NULL. */
+		if (check_aes_parms(MODE_GCM, plaintext_length, (unsigned char *)1,
+				    iv, key_length, key, (unsigned char *)1))
+			return EINVAL;
+	}
 	if (check_gcm_parms(plaintext_length, aad, aad_length, tag, tag_length, iv_length))
 		return EINVAL;
 
+	memset(tmp_tag, 0, sizeof(tmp_tag));
+
 	function_code = aes_directed_fc(key_length, direction);
 	if (direction) {
 		/* encrypt & generate */
@@ -1825,9 +2084,16 @@ unsigned int ica_aes_gcm_intermediate(unsigned char *plaintext,
 		return EACCES;
 #endif /* ICA_FIPS */
 
-	if (check_aes_parms(MODE_GCM, plaintext_length, plaintext, cb, key_length,
-			    key, ciphertext))
-		return EINVAL;
+	if (plaintext_length != 0) {
+		if (check_aes_parms(MODE_GCM, plaintext_length, plaintext, cb, key_length,
+				    key, ciphertext))
+			return EINVAL;
+	} else {
+		/* If only aad is processed (ghash), pt/ct may be NULL. */
+		if (check_aes_parms(MODE_GCM, plaintext_length, (unsigned char *)1,
+				    cb, key_length, key, (unsigned char *)1))
+			return EINVAL;
+	}
 	if (check_gcm_parms(plaintext_length, aad, aad_length, tag, tag_length,
 				iv_length_dummy))
 		return EINVAL;
@@ -1884,6 +2150,169 @@ unsigned int ica_aes_gcm_last( unsigned char *icb,
 	return 0;
 }
 
+/*************************************************************************************
+ *
+ *                                     GCM(2) API
+ */
+
+kma_ctx* ica_aes_gcm_kma_ctx_new(void)
+{
+	kma_ctx* ctx = malloc(sizeof(kma_ctx));
+	if (!ctx)
+		return NULL;
+
+	memset(ctx, 0, sizeof(kma_ctx));
+
+	return ctx;
+}
+
+int ica_aes_gcm_kma_init(unsigned int direction,
+		const unsigned char *iv, unsigned int iv_length,
+		const unsigned char *key, unsigned int key_length,
+		kma_ctx* ctx)
+{
+	int rc = 0;
+	unsigned long function_code = aes_directed_fc(key_length, direction);
+
+	/* Check for obvious errors */
+	if (!ctx || !key || iv_length == 0 || !is_valid_aes_key_length(key_length) ||
+		!is_valid_direction(direction)) {
+		return EINVAL;
+	}
+
+	memset(ctx, 0, sizeof(kma_ctx));
+	ctx->version = 0x00;
+	ctx->direction = direction;
+	ctx->key_length = key_length;
+	ctx->iv = (unsigned char*)iv;
+	ctx->iv_length = iv_length;
+	memcpy(&(ctx->key), key, key_length);
+
+	/* Calculate subkey_h and j0 depending on iv_length */
+	if (*s390_kma_functions[function_code].enabled && iv_length == GCM_RECOMMENDED_IV_LENGTH) {
+		/* let KMA provide the subkey_h, j0 = iv || 00000001 */
+		memcpy(&(ctx->j0), iv, iv_length);
+		ctx->cv = 1;
+		unsigned int* cv = (unsigned int*)&(ctx->j0[GCM_RECOMMENDED_IV_LENGTH]);
+		*cv = 1;
+	} else {
+		/* Calculate subkey H and initial counter, based on iv */
+		rc = s390_aes_ecb(UNDIRECTED_FC(function_code),
+				AES_BLOCK_SIZE, zero_block,
+				(unsigned char*)key, (unsigned char*)&(ctx->subkey_h));
+		if (rc)
+			return rc;
+		__compute_j0(iv, iv_length, (const unsigned char*)&(ctx->subkey_h),
+				(unsigned char*)&(ctx->j0));
+		unsigned int *cv = (unsigned int*)&(ctx->j0[GCM_RECOMMENDED_IV_LENGTH]);
+		ctx->cv = *cv;
+		ctx->subkey_provided = 1;
+	}
+
+	return rc;
+}
+
+int ica_aes_gcm_kma_update(const unsigned char *in_data,
+		unsigned char *out_data, unsigned long data_length,
+		const unsigned char *aad, unsigned long aad_length,
+		unsigned int end_of_aad, unsigned int end_of_data,
+		kma_ctx* ctx)
+{
+	unsigned int function_code = aes_directed_fc(ctx->key_length, ctx->direction);
+
+#ifdef ICA_FIPS
+	if (fips >> 1)
+		return EACCES;
+#endif /* ICA_FIPS */
+
+	if (data_length > 0 && (!in_data || !out_data))
+		return EFAULT;
+
+	if (!(*s390_kma_functions[function_code].enabled)) {
+
+		if (end_of_aad && end_of_data && !ctx->intermediate) {
+			ctx->done = 1;
+			return s390_aes_gcm_simulate_kma_full(in_data, out_data, data_length,
+									aad, aad_length, ctx);
+		} else {
+			ctx->intermediate = 1;
+			return s390_aes_gcm_simulate_kma_intermediate(in_data, out_data, data_length,
+									aad, aad_length, ctx);
+		}
+
+	} else {
+
+		return s390_aes_gcm_kma(in_data, out_data, data_length,
+								aad, aad_length, end_of_aad, end_of_data, ctx);
+	}
+}
+
+int ica_aes_gcm_kma_get_tag(unsigned char *tag, unsigned int tag_length, const kma_ctx* ctx)
+{
+	int rc=0;
+	unsigned int function_code = aes_directed_fc(ctx->key_length, ctx->direction);
+
+	if (!ctx || !tag || !is_valid_tag_length(tag_length))
+		return EINVAL;
+
+	if (ctx->direction == ICA_DECRYPT)
+		return EFAULT;
+
+	if (!(*s390_kma_functions[function_code].enabled) && !ctx->done) {
+		rc = s390_gcm_last(function_code, (unsigned char*)ctx->j0,
+				ctx->total_aad_length, ctx->total_input_length,
+				(unsigned char*)ctx->tag, AES_BLOCK_SIZE,
+				(unsigned char*)ctx->key, (unsigned char*)ctx->subkey_h);
+		if (rc)
+			return rc;
+	}
+
+	memcpy(tag, ctx->tag, tag_length);
+
+	return 0;
+}
+
+int ica_aes_gcm_kma_verify_tag(const unsigned char* known_tag, unsigned int tag_length, const kma_ctx* ctx)
+{
+	int rc;
+	unsigned int function_code = aes_directed_fc(ctx->key_length, ctx->direction);
+
+	if (!ctx || !known_tag || !is_valid_tag_length(tag_length))
+		return EINVAL;
+
+	if (ctx->direction == ICA_ENCRYPT)
+		return EFAULT;
+
+	if (!(*s390_kma_functions[function_code].enabled) && !ctx->done) {
+		rc = s390_gcm_last(function_code, (unsigned char*)ctx->j0,
+				ctx->total_aad_length, ctx->total_input_length,
+				(unsigned char*)ctx->tag, AES_BLOCK_SIZE,
+				(unsigned char*)ctx->key, (unsigned char*)ctx->subkey_h);
+		if (rc)
+			return rc;
+	}
+
+	if (CRYPTO_memcmp(ctx->tag, known_tag, tag_length) != 0)
+		return EFAULT;
+
+	return 0;
+}
+
+void ica_aes_gcm_kma_ctx_free(kma_ctx* ctx)
+{
+    if (!ctx)
+	return;
+
+    OPENSSL_cleanse((void *)ctx, sizeof(kma_ctx));
+
+    free(ctx);
+}
+
+/**
+ *                             End of GCM(2) API
+ *
+ ***************************************************************************************/
+
 unsigned int ica_get_version(libica_version_info *version_info)
 {
 #ifdef VERSION
