@@ -15,23 +15,25 @@
  *	       Holger Dengler <hd@linux.vnet.ibm.com>
  *	       Ingo Tuchscherer <ingo.tuchscherer.linux.vnet.ibm.com>
  *
- * Copyright IBM Copr. 2007, 2009, 2011, 2013
+ * Copyright IBM Copr. 2007, 2009, 2011, 2013, 2016
  */
 
 #include <stdint.h>
 #include <fcntl.h>
 #include <string.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <errno.h>
 
 #include "fips.h"
 #include "init.h"
 #include "s390_crypto.h"
 
-unsigned int sha1_switch, sha256_switch, sha512_switch, des_switch,
+
+unsigned int sha1_switch, sha256_switch, sha512_switch, sha3_switch, des_switch,
 	     tdes_switch, aes128_switch, aes192_switch, aes256_switch,
 	     prng_switch, tdea128_switch, tdea192_switch, sha512_drng_switch,
-	     msa4_switch, msa5_switch;
+	     msa4_switch, msa5_switch, msa8_switch;
 
 s390_supported_function_t s390_kimd_functions[] = {
 	{SHA_1, S390_CRYPTO_SHA_1, &sha1_switch},
@@ -39,6 +41,12 @@ s390_supported_function_t s390_kimd_functions[] = {
 	{SHA_256, S390_CRYPTO_SHA_256, &sha256_switch},
 	{SHA_384, S390_CRYPTO_SHA_512, &sha512_switch},
 	{SHA_512, S390_CRYPTO_SHA_512, &sha512_switch},
+	{SHA_3_224, S390_CRYPTO_SHA_3_224, &sha3_switch},
+	{SHA_3_256, S390_CRYPTO_SHA_3_256, &sha3_switch},
+	{SHA_3_384, S390_CRYPTO_SHA_3_384, &sha3_switch},
+	{SHA_3_512, S390_CRYPTO_SHA_3_512, &sha3_switch},
+	{SHAKE_128, S390_CRYPTO_SHAKE_128, &sha3_switch},
+	{SHAKE_256, S390_CRYPTO_SHAKE_256, &sha3_switch},
 	{GHASH, S390_CRYPTO_GHASH, &msa4_switch}
 };
 
@@ -82,6 +90,19 @@ s390_supported_function_t s390_ppno_functions[] = {
 	{SHA512_DRNG_SEED, S390_CRYPTO_SHA512_DRNG_SEED, &sha512_drng_switch},
 };
 
+s390_supported_function_t s390_kma_functions[] = {
+	{0, 0, &msa8_switch},	/* DEA not supported */
+	{0, 0, &msa8_switch},
+	{0, 0, &msa8_switch},	/* TDEA not supported */
+	{0, 0, &msa8_switch},
+	{AES_128_GCM_ENCRYPT, S390_CRYPTO_AES_128_GCM_ENCRYPT, &msa8_switch},
+	{AES_128_GCM_DECRYPT, S390_CRYPTO_AES_128_GCM_DECRYPT, &msa8_switch},
+	{AES_192_GCM_ENCRYPT, S390_CRYPTO_AES_192_GCM_ENCRYPT, &msa8_switch},
+	{AES_192_GCM_DECRYPT, S390_CRYPTO_AES_192_GCM_DECRYPT, &msa8_switch},
+	{AES_256_GCM_ENCRYPT, S390_CRYPTO_AES_256_GCM_ENCRYPT, &msa8_switch},
+	{AES_256_GCM_DECRYPT, S390_CRYPTO_AES_256_GCM_DECRYPT, &msa8_switch}
+};
+
 int read_cpuinfo(void)
 {
 	int msa = 0;
@@ -104,19 +125,19 @@ int read_cpuinfo(void)
 int read_facility_bits(void)
 {
 	int msa = 0;
-	unsigned long long facility_bits[2] = {0};
+	unsigned long long facility_bits[3] = {0};
 	struct sigaction oldact;
 	sigset_t oldset;
 	int rc = -1;
 
 	rc = begin_sigill_section(&oldact, &oldset);
 	if (!rc) {
-		rc = __stfle(facility_bits, 2);
+		rc = __stfle(facility_bits, 3);
 		end_sigill_section(&oldact, &oldset);
 	}
 	/* __stfle always returns the no. of double words needed to store the
-	 * facility bits. This quantity is machine dependent. However, we just
-	 * need the first two double words. */
+	 * facility bits. This quantity is machine dependent. With MSA8, we
+	 * need the first three double words. */
 	if(rc >= 2){
 		if(facility_bits[0] & (1ULL << (63 - 17)))
 			msa = 1;
@@ -126,6 +147,23 @@ int read_facility_bits(void)
 			msa = 4;
 		if(facility_bits[0] & (1ULL << (63 - 57)))
 			msa = 5;
+		if (facility_bits[2] & (1ULL << (191 - 146)))
+			msa = 8;
+	}
+
+	/**
+	 * allow specifying the MSA level via environment variable
+	 * to simulate older hardware.
+	 */
+	char* s = getenv("MSA");
+	int env_msa;
+
+	if (s) {
+#ifdef ICA_DEBUG
+		printf("msa from getenv: [%s] \n", s);
+#endif
+		if (sscanf(s, "%d", &env_msa) == 1)
+			msa = env_msa > msa ? msa : env_msa;
 	}
 
 	return msa;
@@ -138,6 +176,7 @@ void set_switches(int msa)
 	unsigned int on = 0;
 	struct sigaction oldact;
 	sigset_t oldset;
+
 	/* The function arrays contain integers. Thus to compute the amount of
 	 * their elements the result of sizeof(*functions) has to be divided by
 	 * sizeof(int).
@@ -147,6 +186,7 @@ void set_switches(int msa)
 	 * is also no distict setting of the switch needed in form
 	 * msa4_switch = 1. */
 
+
 	/* kmc query */
 	memset(mask, 0, sizeof(mask));
 	if (msa) {
@@ -198,6 +238,24 @@ void set_switches(int msa)
 			on = 0;
 		*s390_ppno_functions[n].enabled = on;
 	}
+
+	/* kma query */
+	memset(mask, 0, sizeof(mask));
+	if (8 <= msa) {
+		msa8_switch = 1;
+		if (begin_sigill_section(&oldact, &oldset) == 0) {
+			s390_kma(S390_CRYPTO_QUERY, mask, NULL, NULL, 0, NULL, 0);
+			end_sigill_section(&oldact, &oldset);
+		}
+	}
+	for (n = 0; n < (sizeof(s390_kma_functions) /
+			 sizeof(s390_supported_function_t)); n++) {
+		if (S390_CRYPTO_TEST_MASK(mask, s390_kma_functions[n].hw_fc))
+			on = 1;
+		else
+			on = 0;
+		*s390_kma_functions[n].enabled = on;
+	}
 }
 
 void s390_crypto_switches_init(void)
@@ -229,6 +287,12 @@ libica_func_list_element_int icaList[] = {
  {SHA256, KIMD, SHA_256		, ICA_FLAG_SW, 0},
  {SHA384, KIMD, SHA_512		, ICA_FLAG_SW, 0},
  {SHA512, KIMD, SHA_512		, ICA_FLAG_SW, 0},
+ {SHA3_224, KIMD, SHA_3_224, 0, 0},
+ {SHA3_256, KIMD, SHA_3_256, 0, 0},
+ {SHA3_384, KIMD, SHA_3_384, 0, 0},
+ {SHA3_512, KIMD, SHA_3_512, 0, 0},
+ {SHAKE128, KIMD, SHAKE_128, 0, 0},
+ {SHAKE256, KIMD, SHAKE_256, 0, 0},
  {G_HASH, KIMD, GHASH		,           0, 0},
 
  {DES_ECB,      KMC,  DEA_ENCRYPT, ICA_FLAG_SW, 0},
@@ -253,6 +317,7 @@ libica_func_list_element_int icaList[] = {
  {AES_CMAC,     MSA4, AES_128_ENCRYPT,           0, 0},
  {AES_CCM,      MSA4, AES_128_ENCRYPT, 	         0, 0},
  {AES_GCM,      MSA4, AES_128_ENCRYPT,           0, 0},
+ {AES_GCM_KMA,  MSA8, AES_128_GCM_ENCRYPT,       0, 0},
  {AES_XTS,      MSA4, AES_128_XTS_ENCRYPT,       0, 0},
  {P_RNG,    		ADAPTER, 0, ICA_FLAG_SHW | ICA_FLAG_SW, 0},	// SHW (CPACF) + SW
  {RSA_ME, 		ADAPTER, 0, ICA_FLAG_DHW | ICA_FLAG_SW, 0x0F},	// DHW (CEX) + SW / 512,1024,2048, 4096 bit key length
@@ -317,6 +382,10 @@ int s390_initialize_functionlist() {
 		icaList[x].flags = icaList[x].flags |
 		    ((*s390_ppno_functions[icaList[x].id].enabled)? 4: 0);
 	break;
+	case MSA8:
+		icaList[x].flags = icaList[x].flags |
+		((*s390_kma_functions[icaList[x].id].enabled)? 4: 0);
+	break;
 	default:
 		/* Do nothing. */
 	break;
@@ -378,7 +447,8 @@ int s390_get_functionlist(libica_func_list_element *pmech_list,
 	/* Disable the algorithm in the following cases:
 	 * - We are running in FIPS mode and the algorithm is not FIPS
 	 *   approved.
-	 * - We are in an error state. */
+	 * - We are in an error state.
+	 * */
 	if (((fips & ICA_FIPS_MODE) && !fips_approved(icaList[x].mech_mode_id))
 	    || fips >> 1) {
 		pmech_list[x].flags = 0;
@@ -389,4 +459,3 @@ int s390_get_functionlist(libica_func_list_element *pmech_list,
 
   return 0;
 }
-
